exports.instantiate = async function instantiate(module, imports = {}) {
  const adaptedImports = {
    env: Object.assign(Object.create(globalThis), imports.env || {}, {
      abort(message, fileName, lineNumber, columnNumber) {
        // ~lib/builtins/abort(~lib/string/String | null?, ~lib/string/String | null?, u32?, u32?) => void
        message = __liftString(message >>> 0);
        fileName = __liftString(fileName >>> 0);
        lineNumber = lineNumber >>> 0;
        columnNumber = columnNumber >>> 0;
        (() => {
          // @external.js
          throw Error(`${message} in ${fileName}:${lineNumber}:${columnNumber}`);
        })();
      },
      "Date.now": (
        // ~lib/bindings/dom/Date.now() => f64
        Date.now
      ),
      "console.log"(text) {
        // ~lib/bindings/dom/console.log(~lib/string/String) => void
        text = __liftString(text >>> 0);
        console.log(text);
      },
      seed() {
        // ~lib/builtins/seed() => f64
        return (() => {
          // @external.js
          return Date.now() * Math.random();
        })();
      },
    }),
  };
  const { exports } = await WebAssembly.instantiate(module, adaptedImports);
  const memory = exports.memory || imports.env.memory;
  const adaptedExports = Object.setPrototypeOf({
    generateBoundedHashesFromJSON(user_id, target_id) {
      // assembly/index/generateBoundedHashesFromJSON(~lib/string/String, ~lib/string/String) => ~lib/string/String
      user_id = __retain(__lowerString(user_id) || __notnull());
      target_id = __lowerString(target_id) || __notnull();
      try {
        return __liftString(exports.generateBoundedHashesFromJSON(user_id, target_id) >>> 0);
      } finally {
        __release(user_id);
      }
    },
    generateBucketedConfigForUser(sdkKey, userStr) {
      // assembly/index/generateBucketedConfigForUser(~lib/string/String, ~lib/string/String) => ~lib/string/String
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      userStr = __lowerString(userStr) || __notnull();
      try {
        return __liftString(exports.generateBucketedConfigForUser(sdkKey, userStr) >>> 0);
      } finally {
        __release(sdkKey);
      }
    },
    setPlatformData(platformDataStr) {
      // assembly/index/setPlatformData(~lib/string/String) => void
      platformDataStr = __lowerString(platformDataStr) || __notnull();
      exports.setPlatformData(platformDataStr);
    },
    clearPlatformData(empty) {
      // assembly/index/clearPlatformData(~lib/string/String) => void
      empty = __lowerString(empty) || __notnull();
      exports.clearPlatformData(empty);
    },
    setConfigData(sdkKey, configDataStr) {
      // assembly/index/setConfigData(~lib/string/String, ~lib/string/String) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      configDataStr = __lowerString(configDataStr) || __notnull();
      try {
        exports.setConfigData(sdkKey, configDataStr);
      } finally {
        __release(sdkKey);
      }
    },
    murmurhashV3(key, seed) {
      // assembly/helpers/murmurhash/murmurhashV3(~lib/string/String, u32) => u32
      key = __lowerString(key) || __notnull();
      return exports.murmurhashV3(key, seed) >>> 0;
    },
    murmurhashV3_js(key, seed) {
      // assembly/helpers/murmurhash/murmurhashV3_js(~lib/string/String, u32) => ~lib/string/String
      key = __lowerString(key) || __notnull();
      return __liftString(exports.murmurhashV3_js(key, seed) >>> 0);
    },
    initEventQueue(sdkKey, optionsStr) {
      // assembly/managers/eventQueueManager/initEventQueue(~lib/string/String, ~lib/string/String) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      optionsStr = __lowerString(optionsStr) || __notnull();
      try {
        exports.initEventQueue(sdkKey, optionsStr);
      } finally {
        __release(sdkKey);
      }
    },
    flushEventQueue(sdkKey) {
      // assembly/managers/eventQueueManager/flushEventQueue(~lib/string/String) => ~lib/string/String
      sdkKey = __lowerString(sdkKey) || __notnull();
      return __liftString(exports.flushEventQueue(sdkKey) >>> 0);
    },
    onPayloadSuccess(sdkKey, payloadId) {
      // assembly/managers/eventQueueManager/onPayloadSuccess(~lib/string/String, ~lib/string/String) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      payloadId = __lowerString(payloadId) || __notnull();
      try {
        exports.onPayloadSuccess(sdkKey, payloadId);
      } finally {
        __release(sdkKey);
      }
    },
    onPayloadFailure(sdkKey, payloadId, retryable) {
      // assembly/managers/eventQueueManager/onPayloadFailure(~lib/string/String, ~lib/string/String, bool) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      payloadId = __lowerString(payloadId) || __notnull();
      retryable = retryable ? 1 : 0;
      try {
        exports.onPayloadFailure(sdkKey, payloadId, retryable);
      } finally {
        __release(sdkKey);
      }
    },
    queueEvent(sdkKey, userStr, eventStr) {
      // assembly/managers/eventQueueManager/queueEvent(~lib/string/String, ~lib/string/String, ~lib/string/String) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      userStr = __retain(__lowerString(userStr) || __notnull());
      eventStr = __lowerString(eventStr) || __notnull();
      try {
        exports.queueEvent(sdkKey, userStr, eventStr);
      } finally {
        __release(sdkKey);
        __release(userStr);
      }
    },
    queueAggregateEvent(sdkKey, eventStr, variableVariationMapStr) {
      // assembly/managers/eventQueueManager/queueAggregateEvent(~lib/string/String, ~lib/string/String, ~lib/string/String) => void
      sdkKey = __retain(__lowerString(sdkKey) || __notnull());
      eventStr = __retain(__lowerString(eventStr) || __notnull());
      variableVariationMapStr = __lowerString(variableVariationMapStr) || __notnull();
      try {
        exports.queueAggregateEvent(sdkKey, eventStr, variableVariationMapStr);
      } finally {
        __release(sdkKey);
        __release(eventStr);
      }
    },
    cleanupEventQueue(sdkKey) {
      // assembly/managers/eventQueueManager/cleanupEventQueue(~lib/string/String) => void
      sdkKey = __lowerString(sdkKey) || __notnull();
      exports.cleanupEventQueue(sdkKey);
    },
    eventQueueSize(sdkKey) {
      // assembly/managers/eventQueueManager/eventQueueSize(~lib/string/String) => i32
      sdkKey = __lowerString(sdkKey) || __notnull();
      return exports.eventQueueSize(sdkKey);
    },
    checkNumbersFilterFromJSON(number, filterStr) {
      // assembly/testHelpers/checkNumbersFilterFromJSON(~lib/string/String, ~lib/string/String) => bool
      number = __retain(__lowerString(number) || __notnull());
      filterStr = __lowerString(filterStr) || __notnull();
      try {
        return exports.checkNumbersFilterFromJSON(number, filterStr) != 0;
      } finally {
        __release(number);
      }
    },
    checkVersionFiltersFromJSON(appVersion, filterStr) {
      // assembly/testHelpers/checkVersionFiltersFromJSON(~lib/string/String | null, ~lib/string/String) => bool
      appVersion = __retain(__lowerString(appVersion));
      filterStr = __lowerString(filterStr) || __notnull();
      try {
        return exports.checkVersionFiltersFromJSON(appVersion, filterStr) != 0;
      } finally {
        __release(appVersion);
      }
    },
    checkCustomDataFromJSON(data, filterStr) {
      // assembly/testHelpers/checkCustomDataFromJSON(~lib/string/String | null, ~lib/string/String) => bool
      data = __retain(__lowerString(data));
      filterStr = __lowerString(filterStr) || __notnull();
      try {
        return exports.checkCustomDataFromJSON(data, filterStr) != 0;
      } finally {
        __release(data);
      }
    },
    evaluateOperatorFromJSON(operatorStr, userStr) {
      // assembly/testHelpers/evaluateOperatorFromJSON(~lib/string/String, ~lib/string/String) => bool
      operatorStr = __retain(__lowerString(operatorStr) || __notnull());
      userStr = __lowerString(userStr) || __notnull();
      try {
        return exports.evaluateOperatorFromJSON(operatorStr, userStr) != 0;
      } finally {
        __release(operatorStr);
      }
    },
    decideTargetVariationFromJSON(targetStr, boundedHash) {
      // assembly/testHelpers/decideTargetVariationFromJSON(~lib/string/String, f64) => ~lib/string/String
      targetStr = __lowerString(targetStr) || __notnull();
      return __liftString(exports.decideTargetVariationFromJSON(targetStr, boundedHash) >>> 0);
    },
    doesUserPassRolloutFromJSON(rolloutStr, boundedHash) {
      // assembly/testHelpers/doesUserPassRolloutFromJSON(~lib/string/String | null, f64) => bool
      rolloutStr = __lowerString(rolloutStr);
      return exports.doesUserPassRolloutFromJSON(rolloutStr, boundedHash) != 0;
    },
    testConfigBodyClass(configStr) {
      // assembly/testHelpers/testConfigBodyClass(~lib/string/String) => ~lib/string/String
      configStr = __lowerString(configStr) || __notnull();
      return __liftString(exports.testConfigBodyClass(configStr) >>> 0);
    },
    testDVCUserClass(userStr) {
      // assembly/testHelpers/testDVCUserClass(~lib/string/String) => ~lib/string/String
      userStr = __lowerString(userStr) || __notnull();
      return __liftString(exports.testDVCUserClass(userStr) >>> 0);
    },
    testBucketedUserConfigClass(userConfigStr) {
      // assembly/testHelpers/testBucketedUserConfigClass(~lib/string/String) => ~lib/string/String
      userConfigStr = __lowerString(userConfigStr) || __notnull();
      return __liftString(exports.testBucketedUserConfigClass(userConfigStr) >>> 0);
    },
    testSortObjectsByString(arr, direction) {
      // assembly/testHelpers/testSortObjectsByString(~lib/array/Array<assembly/helpers/arrayHelpers/SortingArrayItem<assembly/testHelpers/TestData>>, ~lib/string/String) => ~lib/array/Array<assembly/testHelpers/TestData>
      arr = __retain(__lowerArray((pointer, value) => { new Uint32Array(memory.buffer)[pointer >>> 2] = __lowerRecord161(value) || __notnull(); }, 162, 2, arr) || __notnull());
      direction = __lowerString(direction) || __notnull();
      try {
        return __liftArray(pointer => __liftRecord160(new Uint32Array(memory.buffer)[pointer >>> 2]), 2, exports.testSortObjectsByString(arr, direction) >>> 0);
      } finally {
        __release(arr);
      }
    },
    testEventQueueOptionsClass(optionsStr) {
      // assembly/types/eventQueueOptions/testEventQueueOptionsClass(~lib/string/String) => ~lib/string/String
      optionsStr = __lowerString(optionsStr) || __notnull();
      return __liftString(exports.testEventQueueOptionsClass(optionsStr) >>> 0);
    },
    testDVCEventClass(eventStr) {
      // assembly/types/dvcEvent/testDVCEventClass(~lib/string/String) => ~lib/string/String
      eventStr = __lowerString(eventStr) || __notnull();
      return __liftString(exports.testDVCEventClass(eventStr) >>> 0);
    },
    testDVCRequestEventClass(eventStr, user_id, featureVarsStr) {
      // assembly/types/dvcEvent/testDVCRequestEventClass(~lib/string/String, ~lib/string/String, ~lib/string/String) => ~lib/string/String
      eventStr = __retain(__lowerString(eventStr) || __notnull());
      user_id = __retain(__lowerString(user_id) || __notnull());
      featureVarsStr = __lowerString(featureVarsStr) || __notnull();
      try {
        return __liftString(exports.testDVCRequestEventClass(eventStr, user_id, featureVarsStr) >>> 0);
      } finally {
        __release(eventStr);
        __release(user_id);
      }
    },
    testPlatformDataClass(dataStr) {
      // assembly/types/platformData/testPlatformDataClass(~lib/string/String) => ~lib/string/String
      dataStr = __lowerString(dataStr) || __notnull();
      return __liftString(exports.testPlatformDataClass(dataStr) >>> 0);
    },
  }, exports);
  function __lowerRecord160(value) {
    // assembly/testHelpers/TestData
    // Hint: Opt-out from lowering as a record by providing an empty constructor
    if (value == null) return 0;
    const pointer = exports.__pin(exports.__new(4, 160));
    new Uint32Array(memory.buffer)[pointer + 0 >>> 2] = __lowerString(value.key) || __notnull();
    exports.__unpin(pointer);
    return pointer;
  }
  function __lowerRecord161(value) {
    // assembly/helpers/arrayHelpers/SortingArrayItem<assembly/testHelpers/TestData>
    // Hint: Opt-out from lowering as a record by providing an empty constructor
    if (value == null) return 0;
    const pointer = exports.__pin(exports.__new(8, 161));
    new Uint32Array(memory.buffer)[pointer + 0 >>> 2] = __lowerString(value.value) || __notnull();
    new Uint32Array(memory.buffer)[pointer + 4 >>> 2] = __lowerRecord160(value.entry) || __notnull();
    exports.__unpin(pointer);
    return pointer;
  }
  function __liftRecord160(pointer) {
    // assembly/testHelpers/TestData
    // Hint: Opt-out from lifting as a record by providing an empty constructor
    if (!pointer) return null;
    return {
      key: __liftString(new Uint32Array(memory.buffer)[pointer + 0 >>> 2]),
    };
  }
  function __liftString(pointer) {
    if (!pointer) return null;
    const
      end = pointer + new Uint32Array(memory.buffer)[pointer - 4 >>> 2] >>> 1,
      memoryU16 = new Uint16Array(memory.buffer);
    let
      start = pointer >>> 1,
      string = "";
    while (end - start > 1024) string += String.fromCharCode(...memoryU16.subarray(start, start += 1024));
    return string + String.fromCharCode(...memoryU16.subarray(start, end));
  }
  function __lowerString(value) {
    if (value == null) return 0;
    const
      length = value.length,
      pointer = exports.__new(length << 1, 1) >>> 0,
      memoryU16 = new Uint16Array(memory.buffer);
    for (let i = 0; i < length; ++i) memoryU16[(pointer >>> 1) + i] = value.charCodeAt(i);
    return pointer;
  }
  function __liftArray(liftElement, align, pointer) {
    if (!pointer) return null;
    const
      memoryU32 = new Uint32Array(memory.buffer),
      dataStart = memoryU32[pointer + 4 >>> 2],
      length = memoryU32[pointer + 12 >>> 2],
      values = new Array(length);
    for (let i = 0; i < length; ++i) values[i] = liftElement(dataStart + (i << align >>> 0));
    return values;
  }
  function __lowerArray(lowerElement, id, align, values) {
    if (values == null) return 0;
    const
      length = values.length,
      buffer = exports.__pin(exports.__new(length << align, 0)) >>> 0,
      header = exports.__pin(exports.__new(16, id)) >>> 0,
      memoryU32 = new Uint32Array(memory.buffer);
    memoryU32[header + 0 >>> 2] = buffer;
    memoryU32[header + 4 >>> 2] = buffer;
    memoryU32[header + 8 >>> 2] = length << align;
    memoryU32[header + 12 >>> 2] = length;
    for (let i = 0; i < length; ++i) lowerElement(buffer + (i << align >>> 0), values[i]);
    exports.__unpin(buffer);
    exports.__unpin(header);
    return header;
  }
  const refcounts = new Map();
  function __retain(pointer) {
    if (pointer) {
      const refcount = refcounts.get(pointer);
      if (refcount) refcounts.set(pointer, refcount + 1);
      else refcounts.set(exports.__pin(pointer), 1);
    }
    return pointer;
  }
  function __release(pointer) {
    if (pointer) {
      const refcount = refcounts.get(pointer);
      if (refcount === 1) exports.__unpin(pointer), refcounts.delete(pointer);
      else if (refcount) refcounts.set(pointer, refcount - 1);
      else throw Error(`invalid refcount '${refcount}' for reference '${pointer}'`);
    }
  }
  function __notnull() {
    throw TypeError("value must not be null");
  }
  return adaptedExports;
}
